"""List the user quotas information via CLI."""
from collections import defaultdict
from typing import TextIO
import argparse
import sys

from inductiva import users, _cli
from inductiva.utils import format_utils


def get_quotas(_, fout: TextIO = sys.stdout):
    """ Lists the user's quotas. 

    Lists all the user's quotas and the quotas left for the user to use.
    """
    table = defaultdict(list)

    for name, quota in users.get_quotas().items():
        table["name"].append(name)
        table["in_use"].append(quota["in_use"])
        table["max_allowed"].append(quota["max_allowed"])

    emph_formatter = format_utils.get_ansi_formatter()

    header_formatters = [
        lambda x: emph_formatter(x.upper(), format_utils.Emphasis.BOLD)
    ]

    table = format_utils.get_tabular_str(table,
                                         header_formatters=header_formatters)

    print(table, file=fout, end="")

    return 0


def register(parser):
    """Register the quotas list command."""

    subparser = parser.add_parser("list",
                                  help="List the user's quotas.",
                                  formatter_class=argparse.RawTextHelpFormatter)

    subparser.description = ("The `inductiva quotas list` command provides "
                             "an overview of your quotas.\n"
                             "It lists all your available quotas as well as the"
                             "quotas left for you to use\n")

    _cli.utils.add_watch_argument(subparser)

    subparser.set_defaults(func=get_quotas)



"""Methods to interact with the user info on the API."""
import json
from typing import Any, Dict, List

from inductiva import api
from inductiva.client import ApiClient
from inductiva.client.apis.tags.users_api import UsersApi


def _fetch_quotas_from_api() -> List[Dict]:
    """Get information about a user's quotas.
    """
    api_config = api.get_api_config()

    with ApiClient(api_config) as client:
        api_instance = UsersApi(client)

        resp = api_instance.get_user_quotas().response
        quotas = json.loads(resp.data.decode("utf-8"))

        return quotas


def get_quotas() -> Dict[str, Dict[str, Any]]:
    """Get the user quotas.

    This function gets a dict with the user quotas.

    Returns:
        Dict with the user quotas.
    """
    quotas = _fetch_quotas_from_api()

    quotas_dict = {quota.pop("name"): quota for quota in quotas}

    return quotas_dict



# coding: utf-8

"""
    InductivaWebAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""

from datetime import date, datetime  # noqa: F401
import decimal  # noqa: F401
import functools  # noqa: F401
import io  # noqa: F401
import re  # noqa: F401
import typing  # noqa: F401
import typing_extensions  # noqa: F401
import uuid  # noqa: F401

import frozendict  # noqa: F401

from inductiva.client import schemas  # noqa: F401


class Cost(
    schemas.DictSchema
):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """


    class MetaOapg:
        required = {
            "start_ts",
            "cost",
            "end_ts",
            "user",
        }
        
        class properties:
            start_ts = schemas.DateTimeSchema
            end_ts = schemas.DateTimeSchema
            cost = schemas.NumberSchema
            user = schemas.StrSchema
            __annotations__ = {
                "start_ts": start_ts,
                "end_ts": end_ts,
                "cost": cost,
                "user": user,
            }
    
    start_ts: MetaOapg.properties.start_ts
    cost: MetaOapg.properties.cost
    end_ts: MetaOapg.properties.end_ts
    user: MetaOapg.properties.user
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["start_ts"]) -> MetaOapg.properties.start_ts: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["end_ts"]) -> MetaOapg.properties.end_ts: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["cost"]) -> MetaOapg.properties.cost: ...
    
    @typing.overload
    def __getitem__(self, name: typing_extensions.Literal["user"]) -> MetaOapg.properties.user: ...
    
    @typing.overload
    def __getitem__(self, name: str) -> schemas.UnsetAnyTypeSchema: ...
    
    def __getitem__(self, name: typing.Union[typing_extensions.Literal["start_ts", "end_ts", "cost", "user", ], str]):
        # dict_instance[name] accessor
        return super().__getitem__(name)
    
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["start_ts"]) -> MetaOapg.properties.start_ts: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["end_ts"]) -> MetaOapg.properties.end_ts: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["cost"]) -> MetaOapg.properties.cost: ...
    
    @typing.overload
    def get_item_oapg(self, name: typing_extensions.Literal["user"]) -> MetaOapg.properties.user: ...
    
    @typing.overload
    def get_item_oapg(self, name: str) -> typing.Union[schemas.UnsetAnyTypeSchema, schemas.Unset]: ...
    
    def get_item_oapg(self, name: typing.Union[typing_extensions.Literal["start_ts", "end_ts", "cost", "user", ], str]):
        return super().get_item_oapg(name)
    

    def __new__(
        cls,
        *_args: typing.Union[dict, frozendict.frozendict, ],
        start_ts: typing.Union[MetaOapg.properties.start_ts, str, datetime, ],
        cost: typing.Union[MetaOapg.properties.cost, decimal.Decimal, int, float, ],
        end_ts: typing.Union[MetaOapg.properties.end_ts, str, datetime, ],
        user: typing.Union[MetaOapg.properties.user, str, ],
        _configuration: typing.Optional[schemas.Configuration] = None,
        **kwargs: typing.Union[schemas.AnyTypeSchema, dict, frozendict.frozendict, str, date, datetime, uuid.UUID, int, float, decimal.Decimal, None, list, tuple, bytes],
    ) -> 'Cost':
        return super().__new__(
            cls,
            *_args,
            start_ts=start_ts,
            cost=cost,
            end_ts=end_ts,
            user=user,
            _configuration=_configuration,
            **kwargs,
        )



"""List the tasks information via CLI."""
from typing import TextIO
import argparse
import sys

from inductiva import tasks, _cli, projects
from inductiva.utils import format_utils
from inductiva.client import models


def list_tasks(args, fout: TextIO = sys.stdout):
    """ List the last user's tasks. 

    Lists based on the flags (task_id, last_n or project_name).
    It can print the last N tasks (default value is 5)
    or a single task with a specific ID
    or all tasks of a project.
    """
    if args.project_name is not None:
        print(f"Showing tasks for project: {args.project_name}.")
        task_list = projects.Project(args.project_name).get_tasks()
    elif args.task_id is not None:
        task_list = [tasks.Task(args.task_id)]
    else:
        last_n = 5 if args.last_n is None else args.last_n
        task_list = tasks.get(last_n=last_n)

    if not task_list:
        print("No tasks found.", file=fout)
        return 1

    table_dict = tasks.to_dict(task_list)

    emph_formatter = format_utils.get_ansi_formatter()

    def color_formater(status):
        if status == models.TaskStatusCode.SUCCESS:
            return emph_formatter(status, format_utils.Emphasis.GREEN)
        elif status in tasks.Task.FAILED_STATUSES:
            return emph_formatter(status, format_utils.Emphasis.RED)
        return status

    formatters = {
        "Submitted": [format_utils.datetime_formatter,],
        "Started": [format_utils.datetime_formatter,],
        "Status": [color_formater]
    }

    header_formatters = [
        lambda x: emph_formatter(x.upper(), format_utils.Emphasis.BOLD)
    ]

    print(format_utils.get_tabular_str(table_dict,
                                       formatters=formatters,
                                       header_formatters=header_formatters),
          file=fout,
          end="")

    return 0


def register(parser):
    """Register the list user's tasks command."""

    subparser = parser.add_parser("list",
                                  help="List tasks.",
                                  formatter_class=argparse.RawTextHelpFormatter)

    subparser.description = ("The `inductiva tasks list` command provides "
                             "an overview of your tasks on the platform.\n"
                             "It lists the most recent tasks or a specific "
                             "task by ID.\n"
                             "You can control the number of tasks listed with "
                             "the '-n' or '--last-n' option.\n")

    _cli.utils.add_watch_argument(subparser)

    group = subparser.add_mutually_exclusive_group()
    group.add_argument("-n",
                       "--last-n",
                       type=int,
                       help="List last N tasks. Default: 5.")
    group.add_argument("-id",
                       "--task-id",
                       type=str,
                       help="List a task with a specific ID.")
    group.add_argument("-p",
                       "--project-name",
                       type=str,
                       help="List the tasks of a project.")

    subparser.set_defaults(func=list_tasks)



# do not import all endpoints into this module because that uses a lot of memory and stack frames
# if you need the ability to import all endpoints from this module, import them with
# from inductiva.client.apis.path_to_api import path_to_api

import enum


class PathValues(str, enum.Enum):
    TASKS_AUTH = "/tasks/auth"
    TASKS_SUBMIT = "/tasks/submit"
    TASKS_TASK_ID_INPUT_UPLOAD_URL = "/tasks/{task_id}/input_upload_url"
    TASKS_TASK_ID_INPUT_UPLOADED = "/tasks/{task_id}/input_uploaded"
    TASKS_TASK_ID_INPUT = "/tasks/{task_id}/input"
    TASKS_TASK_ID = "/tasks/{task_id}"
    TASKS = "/tasks"
    TASKS_TASK_ID_STATUS = "/tasks/{task_id}/status"
    TASKS_TASK_ID_OUTPUT_LIST = "/tasks/{task_id}/output/list"
    TASKS_TASK_ID_DOWNLOAD_OUTPUT_URL = "/tasks/{task_id}/download_output_url"
    TASKS_TASK_ID_OUTPUT = "/tasks/{task_id}/output"
    TASKS_TASK_ID_RESUBMIT = "/tasks/{task_id}/resubmit"
    TASKS_TASK_ID_KILL = "/tasks/{task_id}/kill"
    TASKS_TASK_ID_DISABLE_LOGS = "/tasks/{task_id}/disable_logs"
    ADMIN_USERS = "/admin/users"
    ADMIN_USERS_EMAIL_API_KEY = "/admin/users/{email}/api_key"
    ADMIN_USERS_EMAIL = "/admin/users/{email}"
    ADMIN_USERS_EMAIL_EXPIRY_TS = "/admin/users/{email}/expiry_ts"
    ADMIN_USERS_USERNAME_TASKS = "/admin/users/{username}/tasks"
    ADMIN_GROUPS = "/admin/groups"
    ADMIN_GROUPS_ACTIVE = "/admin/groups/active"
    ADMIN_GROUPS_DEFAULT = "/admin/groups/default"
    ADMIN_GROUPS_DEFAULT_MACHINE_GROUP_ID = "/admin/groups/default/{machine_group_id}"
    ADMIN_PROVIDERS = "/admin/providers"
    ADMIN_PROVIDERS_PROVIDER_ID = "/admin/providers/{provider_id}"
    ADMIN_ACTIVE_TASKS = "/admin/active_tasks"
    ADMIN_USERS_USERNAME_COST = "/admin/users/{username}/cost"
    ADMIN_EXECUTERTRACKER_TOKEN = "/admin/executer-tracker/token"
    EXECUTERTRACKER_REGISTER = "/executer-tracker/register"
    EXECUTERTRACKER_MACHINE_ID = "/executer-tracker/{machine_id}"
    EXECUTERTRACKER_MACHINE_ID_TASK = "/executer-tracker/{machine_id}/task"
    EXECUTERTRACKER_MACHINE_ID_TASK_TASK_ID_ACK = "/executer-tracker/{machine_id}/task/{task_id}/ack"
    EXECUTERTRACKER_MACHINE_ID_TASK_TASK_ID_MESSAGE = "/executer-tracker/{machine_id}/task/{task_id}/message"
    EXECUTERTRACKER_MACHINE_ID_TASK_TASK_ID_MESSAGE_UNBLOCK = "/executer-tracker/{machine_id}/task/{task_id}/message/unblock"
    EXECUTERTRACKER_MACHINE_ID_EVENT = "/executer-tracker/{machine_id}/event"
    EXECUTERTRACKER_MACHINE_ID_TASK_TASK_ID_DOWNLOAD_INPUT_URL = "/executer-tracker/{machine_id}/task/{task_id}/download_input_url"
    EXECUTERTRACKER_MACHINE_ID_TASK_TASK_ID_UPLOAD_OUTPUT_URL = "/executer-tracker/{machine_id}/task/{task_id}/upload_output_url"
    EXECUTERTRACKER_MACHINE_ID_TASK_TASK_ID_METRIC = "/executer-tracker/{machine_id}/task/{task_id}/metric"
    COMPUTE_GROUP = "/compute/group"
    COMPUTE_TYPE = "/compute/type"
    COMPUTE_GROUP_START = "/compute/group/start"
    COMPUTE_GROUP_MG_ID = "/compute/group/{mg_id}"
    COMPUTE_PRICE = "/compute/price"
    COMPUTE_GROUPS = "/compute/groups"
    COMPUTE_GROUP_STATUS = "/compute/group_status"
    COMPUTE_MACHINE_TYPES = "/compute/machine_types"
    COMPUTE_GROUP_NAME = "/compute/group/{name}"
    STORAGE_SIZE = "/storage/size"
    STORAGE_CONTENTS = "/storage/contents"
    VERSION = "/version"
    VERSIONCHECK = "/version-check"
    USERS_COST = "/users/cost"
    USERS_QUOTAS = "/users/quotas"
    USERS_ME = "/users/me"
    PROJECTS = "/projects"
    PROJECTS_NAME = "/projects/{name}"
